#!/bin/bash
set -o nounset
set -o errexit
#-------------------------------------------------------------------------------
## DESCRIPTION
## Downloads FASTQ data from the study GSE223043 and prepare files and
## directories to run a miRNA-Seq analysis with myBrain-Seq.
##
## Script arguments
##    $1 : working directory for the analysis replication.
## 
## Script parameters
##    n_parallel_downloads: number of threads in the download.
##    srs_codes           : path to a file with SRR codes, one per line.
##
## Dependencies:
##    sra-toolkit::fasternq-dump
##    edirect::esearch
#-------------------------------------------------------------------------------

# parameters
n_parallel_downloads=6
url_bowtie_index='http://static.sing-group.org/software/myBrainSeq/downloads/GRCh38_noalt_as.zip'
url_mirbase='https://www.mirbase.org/ftp/22.1/genomes/hsa.gff3'
bowtie_rootname='GRCh38_noalt_as'

# function to remove double slashes from paths
function remove_double_slashes {
  echo ${1} | tr -s '/'
}

#-------------------------------------------------------------------------------
#                    INITIALIZATION OF THE WORKING DIR
#-------------------------------------------------------------------------------
echo 'Preparing the working directory'
# PATHS
# directories of the working-dir
wd=$(remove_double_slashes "${1}/")
input=$(remove_double_slashes "${wd}/input")
output=$(remove_double_slashes "${wd}/output")
references=$(remove_double_slashes "${input}/references")
genome_index=$(remove_double_slashes "${references}/bowtie_index")
fastq_dir=$(remove_double_slashes "${input}/fastq")
# working files
conditions=$(remove_double_slashes "${input}/conditions_file.txt")
contrast=$(remove_double_slashes "${input}/contrast_file.txt")
ref_mirbase="$(remove_double_slashes "${references}/hsa.gff3")"
compi_parameters="${input}/compi.parameters"
# temporary files
temp_dir="${fastq_dir}/temp_GSE223043"
metadata_SRR="${temp_dir}/metadata_SRR.xml"
srr_codes="${temp_dir}/STUDY-003_SRR-codes.txt"

# DIRECTORY AND FILES CREATION
# creation of directories
mkdir -p "${input}" "${output}" "${fastq_dir}" "${references}" \
"${temp_dir}" "${genome_index}"
# creation of conditions_file and contrast_file
touch "${conditions}"; touch "${contrast}"

#-------------------------------------------------------------------------------
#                      DOWNLOAD THE FASTQ FILES FROM GEO
#-------------------------------------------------------------------------------
echo 'Downloading the fastq files of the study GSE223043'
# PREPARES A FILE WITH THE SRR CODES OF MR AND MNR SAMPLES
# download the metadata of the study
docker run --rm \
	-v ${wd}:${wd} \
	pegi3s/entrez-direct \
  bash -c "esearch -db sra -query \"PRJNA924754\" | efetch -format docsum > ${metadata_SRR}"
# parse the xml file to find SRR accession numbers
docker run --rm \
  -v ${wd}:${wd} \
	pegi3s/entrez-direct \
  bash -c "cat ${metadata_SRR} | \
  xtract -pattern 'DocumentSummary' -element 'Title' -element 'Run@acc' | \
  grep 'Multiple' | cut -f2 > ${srr_codes}"

# DOWNLOAD THE SAMPLES
# download samples
docker run --rm \
	-v ${wd}:${wd} \
	pegi3s/sratoolkit:latest\
	bash -c "cat ${srr_codes} | xargs -I {} fasterq-dump {} -O \"${fastq_dir}\" \
  -t \"${temp_dir}\" --threads \"${n_parallel_downloads}\""

#-------------------------------------------------------------------------------
#                    DOWNLOAD THE BIOLOGICAL REFERENCES
#-------------------------------------------------------------------------------
echo 'Preparing the biological references'
# BOWTIE INDEX (GRCh38)
# download
echo "Downloading Bowtie index for GRCh38"
cd "${genome_index}"
curl "${url_bowtie_index}" > bowtie.zip
# decompress and remove
unzip -j "bowtie.zip"
rm -f "bowtie.zip"

# MIRBASE ANNOTATIONS (GRCh38)
# test if mirbase annotations are for GRCh38
echo "Downloading miRBase annotations"
# download mirbase annotations
curl "${url_mirbase}" > "${ref_mirbase}"

#-------------------------------------------------------------------------------
#                          FILL MYBRAIN-SEQ FILES
#-------------------------------------------------------------------------------
echo "Preparing the files for the analysis with myBrain-Seq"
# COMPI.PARAMETERS FILE
printf "workingDir=${wd}
conditions=${conditions}
contrast=${contrast}
fastqDir=${fastq_dir}
adapter=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC
bwtIndex=${genome_index}/${bowtie_rootname}
gffFile=${ref_mirbase}
organism=Homo sapiens
" > ${compi_parameters}

# MYBRAIN-SEQ CONDITIONS FILE
# fill content
echo 'name	condition	label	lab_isolation	sex	anydrug	time	zepinas	other_antipsy
SRR23099018	MNR	Non-responder	13/05/20	F	no	A	1	0
SRR23099017	MNR	Non-responder	13/05/20	F	no	D	1	0
SRR23099016	MNR	Non-responder	13/05/20	M	yes	A	0	0
SRR23099015	MNR	Non-responder	13/05/20	M	yes	D	0	0
SRR23099014	MNR	Non-responder	13/05/20	M	yes	A	1	0
SRR23099013	MNR	Non-responder	14/05/20	M	yes	D	1	0
SRR23099012	MNR	Non-responder	14/05/20	F	yes	A	0	1
SRR23099011	MNR	Non-responder	14/05/20	F	yes	D	0	1
SRR23099010	MNR	Non-responder	14/05/20	F	yes	A	0	0
SRR23099009	MNR	Non-responder	15/05/20	F	yes	D	0	0
SRR23099008	MNR	Non-responder	14/05/20	M	yes	A	0	1
SRR23099007	MNR	Non-responder	15/05/20	M	yes	D	0	1
SRR23099005	MNR	Non-responder	18/05/20	F	yes	D	0	0
SRR23099004	MNR	Non-responder	15/05/20	M	yes	A	1	0
SRR23099109	MNR	Non-responder	15/05/20	M	no	A	1	0
SRR23099108	MNR	Non-responder	18/05/20	M	no	A	1	0
SRR23099107	MNR	Non-responder	18/05/20	M	yes	A	1	0
SRR23099106	MNR	Non-responder	18/05/20	F	yes	A	1	0
SRR23099105	MNR	Non-responder	19/05/20	M	yes	A	1	0
SRR23099104	MNR	Non-responder	19/05/20	M	yes	A	1	0
SRR23099103	MNR	Non-responder	19/05/20	M	yes	A	1	0
SRR23099102	MNR	Non-responder	20/05/20	F	yes	A	1	0
SRR23099101	MNR	Non-responder	20/05/20	F	no	A	0	1
SRR23099100	MNR	Non-responder	18/05/20	F	no	D	0	1
SRR23099099	MNR	Non-responder	20/05/20	M		A	0	0
SRR23099098	MNR	Non-responder	19/05/20	M		D	0	0
SRR23099097	MNR	Non-responder	20/05/20	F	no	A	0	0
SRR23099096	MNR	Non-responder	19/05/20	F	no	D	0	0
SRR23099095	MNR	Non-responder	20/05/20	M	no	A	1	0
SRR23099094	MNR	Non-responder	20/05/20	M	no	D	1	0
SRR23099093	MNR	Non-responder	20/05/20	M	yes	A	1	0
SRR23099092	MNR	Non-responder	20/05/20	M	yes	D	1	0
SRR23099091	MR	Responder	13/05/20	M	yes	A	0	1
SRR23099090	MR	Responder	13/05/20	M	yes	D	0	1
SRR23099089	MR	Responder	13/05/20	M	no	A	0	1
SRR23099088	MR	Responder	13/05/20	M	no	D	0	1
SRR23099087	MR	Responder	13/05/20	F	yes	A	0	1
SRR23099085	MR	Responder	13/05/20	F	yes	D	0	1
SRR23098995	MR	Responder	14/05/20	F	no	A	0	1
SRR23098994	MR	Responder	14/05/20	F	no	D	0	1
SRR23098993	MR	Responder	14/05/20	F	no	A	1	0
SRR23098992	MR	Responder	14/05/20	F	no	D	1	0
SRR23098991	MR	Responder	14/05/20	M	yes	A	0	0
SRR23098990	MR	Responder	14/05/20	M	yes	D	0	0
SRR23098989	MR	Responder	15/05/20	F	no	A	1	0
SRR23098988	MR	Responder	15/05/20	F	no	D	1	0
SRR23098987	MR	Responder	15/05/20	M	yes	A	1	0
SRR23098986	MR	Responder	15/05/20	M	yes	D	1	0
SRR23098985	MR	Responder	15/05/20	F	no	A	0	1
SRR23098984	MR	Responder	15/05/20	F	no	D	0	1
SRR23098983	MR	Responder	18/05/20	M	yes	A	0	1
SRR23098982	MR	Responder	18/05/20	M	yes	D	0	1
SRR23098981	MR	Responder	18/05/20	M	yes	A	0	1
SRR23098980	MR	Responder	18/05/20	M	yes	D	0	1
SRR23098979	MR	Responder	18/05/20	M	yes	A	0	1
SRR23098978	MR	Responder	18/05/20	M	yes	D	0	1
SRR23098977	MR	Responder	19/05/20	M	yes	A	0	1
SRR23098976	MR	Responder	19/05/20	M	yes	D	0	1
SRR23098975	MR	Responder	19/05/20	M	yes	A	0	1
SRR23098974	MR	Responder	19/05/20	M	yes	D	0	1
SRR23098973	MR	Responder	19/05/20	M	yes	A	0	0
SRR23098972	MR	Responder	19/05/20	M	yes	D	0	0
SRR23099057	MR	Responder	20/05/20	M	yes	A	1	0
SRR23099056	MR	Responder	20/05/20	M	yes	D	1	0
SRR23099055	MR	Responder	20/05/20	M	no	A	0	1
SRR23099054	MR	Responder	20/05/20	M	no	D	0	1
SRR23099053	MR	Responder	20/05/20	M	yes	A	1	0
SRR23099052	MR	Responder	20/05/20	M	yes	D	1	0
SRR23099051	MR	Responder	20/05/20	M	yes	A	0	0
SRR23099050	MR	Responder	20/05/20	M	yes	D	0	0
' > "${conditions}"

# CONTRAST FILE
echo 'name
"Responder-Resistant" = "MR-MNR"' > "${contrast}"

#-------------------------------------------------------------------------------
#                      PREPARE MYBRAIN-SEQ ANALYSIS
#-------------------------------------------------------------------------------
# BUILD MYBRAIN-SEQ RUNNER
COMPI_PARAMETERS=/path/to/compi.parameters

docker run --rm --entrypoint /init-working-dir/make_run-sh.sh \
  -v ${wd}:${wd} \
  -v ${compi_parameters}:${compi_parameters} \
  singgroup/my-brain-seq ${compi_parameters}

# remove temporary files
rm -rf "${temp_dir}"